#!/usr/bin/perl
use v5.14;
use autodie;

use strict;
@ARGV == 1 or die "Uso: $0 <archivo LaTeX a procesar\n";
 
my $nombre_archivo_entrada = shift;
 
-f $nombre_archivo_entrada or die "ERROR: No encuentro $nombre_archivo_entrada\n";
 
 
## Lectura del archivo
open my $ENTRADA, '<', $nombre_archivo_entrada;
undef $/;                                                                       # modo 'slurp'
my $archivo_entrada = <$ENTRADA>;
close   $ENTRADA;
 
 
## Partición del documento
my($cabeza,$cuerpo,$final) = $archivo_entrada =~ m/\A (.+? ^\\begin{document}) \s* (.+?) \s* (^ \\end{document}) \s* \z/msx;
 
 
## Cambios a realizar
my %cambios = (
	'\pspicture'                => '\TRICKS',
	'\endpspicture'             => '\ENDTRICKS',
 
	'\begin{pspicture'          => '\begin{TRICKS',
	'\end{pspicture'            => '\end{TRICKS',
 
	'\postscript'               => '\POSTRICKS',
 
	'\begin{postscript}'        => '\begin{POSTRICKS}',
	'\end{postscript}'          => '\end{POSTRICKS}',
);
 
 
## Variables y constantes
my $no_del = "\0";
my $del    = $no_del;
 
## Reglas
my $llaves      = qr/\{ .+? \}                                                                  /x;
my $no_corchete = qr/(?:\[ .+? \])?                                                             /x;
my $delimitador = qr/\{ (?<del>.+?) \}                                                          /x;
my $verb        = qr/verb [*]?                                                                  /ix;
my $lst         = qr/lstinline (?!\*) $no_corchete                                              /ix;
my $mint        = qr/mint      (?!\*) $no_corchete $llaves                                      /ix;
my $marca       = qr/\\ (?:$verb | $lst | $mint) (\S) .+? \g{-1}                                /x;
my $comentario  = qr/^ \s* \%+ .+? $                                                            /mx;
my $definedel   = qr/\\ (?:   DefineShortVerb | lstMakeShortInline  ) $no_corchete $delimitador /ix;
my $indefinedel = qr/\\ (?: UndefineShortVerb | lstDeleteShortInline) $llaves                   /ix;
 
 
## Cambiar
while ($cuerpo =~
	/   $marca
	|   $comentario
	|   $definedel
	|   $indefinedel
	|   $del .+? $del                                                           # delimitado
	/gimx) {
 
	my($pos_inicial, $pos_final) = ($-[0], $+[0]);                              # posiciones
	my $encontrado = ${^MATCH};                                                 # lo encontrado
 
	given ($encontrado) {
		when (/$definedel/) {                                                   # definimos delimitador
			$del = $+{del};
			$del = "\Q$+{del}" if substr($del,0,1) ne '\\';                     # es necesario "escapar" el delimitador
		}
		when (/$indefinedel/) {                                                 # indefinimos delimitador
			$del = $no_del;
		}
		default {                                                               # Aquí se hacen los cambios
			while (my($busco, $cambio) = each %cambios) {
 
				$encontrado =~ s/\Q$busco\E/$cambio/g;                          # es necesario escapar $busco, ya que contiene caracteres extraños
			}
 
			substr $cuerpo, $pos_inicial, $pos_final-$pos_inicial, $encontrado; # insertamos los nuevos cambios

			pos($cuerpo)= $pos_inicial + length $encontrado;                    # reposicionamos la siguiente búsqueda
		}
	}
}
 

## Escritura del resultado
open my $SALIDA, '>', "test-out2.tex";
print   $SALIDA "$cabeza\n$cuerpo\n$final\n";
close   $SALIDA;
 
__END__
